/*
 * This file is part of John the Ripper password cracker,
 * Copyright (c) 1996-98 by Solar Designer
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "arch.h"
#include "misc.h"
#include "params.h"
#include "path.h"
#include "memory.h"
#include "list.h"
#include "tty.h"
#include "signals.h"
#include "common.h"
#include "formats.h"
#include "loader.h"
#include "logger.h"
#include "status.h"
#include "options.h"
#include "config.h"
#include "bench.h"
#include "charset.h"
#include "wordlist.h"
#include "single.h"
#include "inc.h"
#include "external.h"

#if CPU_DETECT
extern int CPU_detect();
#endif

extern struct fmt_main fmt_DES, fmt_BSDI, fmt_MD5, fmt_BF;

extern int unshadow(int argc, char **argv);

static struct db_main database;

static void john_register_one(struct fmt_main *format)
{
	if (options.format)
	if (strcmp(options.format, format->params.label)) return;

	fmt_register(format);
}

static void john_register_all()
{
	if (options.format) strlwr(options.format);

	john_register_one(&fmt_DES);
	john_register_one(&fmt_BSDI);
	john_register_one(&fmt_MD5);
	john_register_one(&fmt_BF);

	if (!fmt_list) {
		puts("Unknown ciphertext format name requested");
		error();
	}
}

static void john_load()
{
	struct list_entry *current;

	if (options.flags & FLG_EXTERNAL_CHK)
		ext_init(options.external);

	if (options.flags & FLG_MAKECHARS_CHK) {
		options.loader.flags |= DB_CRACKED;
		ldr_init_database(&database, &options.loader);

		if (options.flags & FLG_PASSWD) {
			ldr_show_pot_file(&database, LOG_NAME);

			database.options->flags |= DB_PLAINTEXTS;
			if ((current = options.passwd->head))
			do {
				ldr_show_pw_file(&database, current->data);
			} while ((current = current->next));
		} else {
			database.options->flags |= DB_PLAINTEXTS;
			ldr_show_pot_file(&database, LOG_NAME);
		}

		return;
	}

	if (options.flags & FLG_PASSWD) {
		if (options.flags & FLG_SHOW_CHK) {
			options.loader.flags |= DB_CRACKED;
			ldr_init_database(&database, &options.loader);

			ldr_show_pot_file(&database, LOG_NAME);

			if ((current = options.passwd->head))
			do {
				ldr_show_pw_file(&database, current->data);
			} while ((current = current->next));

			printf("%s%d password%s cracked, %d left\n",
				database.guess_count ? "\n" : "",
				database.guess_count,
				database.guess_count != 1 ? "s" : "",
				database.password_count -
				database.guess_count);

			return;
		}

		if (options.flags & FLG_SINGLE_CHK)
			options.loader.flags |= DB_WORDS;
		else
		if (mem_saving_level)
			options.loader.flags &= ~DB_LOGIN;
		ldr_init_database(&database, &options.loader);

		if ((current = options.passwd->head))
		do {
			ldr_load_pw_file(&database, current->data);
		} while ((current = current->next));

		ldr_load_pot_file(&database, LOG_NAME);

		ldr_fix_database(&database);

		printf("Loaded %d password%s%s",
			database.password_count,
			database.password_count != 1 ? "s" : "",
			database.password_count ? "" : ", exiting...");
		if (database.password_count > 1) {
			printf(" with ");
			printf(database.salt_count != 1 ? "%d" : "no",
				database.salt_count);
			printf(" different salts");
		}
		if (database.password_count)
			printf(" (%s [%s])\n",
				database.format->params.format_name,
				database.format->params.algorithm_name);
		else
			putchar('\n');

		if ((options.flags & FLG_PWD_REQ) && !database.salts) exit(0);
	}
}

static void john_init(int argc, char **argv)
{
	puts("\nJohn the Ripper  Version " JOHN_VERSION
		"  Copyright (c) 1996-98 by Solar Designer\n");

#if CPU_DETECT
	if (!CPU_detect()) {
#if CPU_REQ
		puts("Sorry, " CPU_NAME " is required");
		error();
#endif
	}
#endif

	path_init(argv);
	cfg_init(CFG_NAME);
	opt_init(argc, argv);

	john_register_all();
	common_init();

	sig_init();

	john_load();
}

static void john_run()
{
	if (options.flags & FLG_TEST_CHK)
		benchmark_all();
	else
	if (options.flags & FLG_MAKECHARS_CHK)
		do_makechars(&database, options.charset);
	else
	if (options.flags & FLG_CRACKING_CHK) {
		log_init(LOG_NAME);
		tty_init();

		if (options.flags & FLG_WORDLIST_CHK) {
			status_init(get_wordlist_progress);
			do_wordlist_crack(&database, options.wordlist,
				(options.flags & FLG_RULES) != 0);
		} else
		if (options.flags & FLG_SINGLE_CHK) {
			status_init(get_single_progress);
			do_single_crack(&database);
		} else
		if (options.flags & FLG_INC_CHK) {
			status_init(NULL);
			do_incremental_crack(&database, options.charset);
		} else
		if (options.flags & FLG_EXTERNAL_CHK) {
			status_init(NULL);
			do_external_crack(&database);
		}

		status_print();
		tty_done();
		log_done();
	}
}

static void john_done()
{
	path_done();

	check_abort();
}

int main(int argc, char **argv)
{
	char *name;

#ifdef __DJGPP__
	if (!--argc) return 1;
	if ((name = strrchr(argv[0], '/')))
		strcpy(name + 1, argv[1]);
	name = argv[1];
	argv[1] = argv[0];
	argv++;
#else
	if ((name = strrchr(argv[0], '/')))
		name++;
	else
		name = argv[0];
#endif

#ifdef __CYGWIN32__
	if (!strcmp(strlwr(name) + strlen(name) - 4, ".exe"))
		name[strlen(name) - 4] = 0;
#endif

	if (!strcmp(name, "john")) {
		john_init(argc, argv);
		john_run();
		john_done();

		return 0;
	}

	if (!strcmp(name, "unshadow"))
		return unshadow(argc, argv);

	puts("Sorry, I can't find myself");
	return 1;
}
